@@ -38,6 +38,15 @@ DEFINE_TYPE_NAME(long double);
 
 typedef boost::mpl::list<float,double,long double>  test_types;
 
+// Apple GCC 4.0 uses the "double double" format for its long double,
+// which means that epsilon is VERY small but useless for
+// comparisons. So, don't do those comparisons.
+#if defined(__APPLE_CC__) && defined(__GNUC__) && __GNUC__ == 4
+typedef boost::mpl::list<float,double>  near_eps_test_types;
+#else
+typedef boost::mpl::list<float,double,long double>  near_eps_test_types;
+#endif
+
 #if BOOST_WORKAROUND(__GNUC__, < 3)
     // gcc 2.x ignores function scope using declarations,
     // put them in the scope of the enclosing namespace instead:
@@ -734,10 +743,13 @@ boost::unit_test_framework::test_suite *    init_unit_test_suite(int, char *[])
 #define    BOOST_OCTONION_COMMON_GENERATOR(fct) \
     test->add(BOOST_TEST_CASE_TEMPLATE(fct##_test, test_types));
     
+#define    BOOST_OCTONION_COMMON_GENERATOR_NEAR_EPS(fct) \
+    test->add(BOOST_TEST_CASE_TEMPLATE(fct##_test, near_eps_test_types));
+    
     
 #define    BOOST_OCTONION_TEST                      \
     BOOST_OCTONION_COMMON_GENERATOR(multiplication) \
-    BOOST_OCTONION_COMMON_GENERATOR(exp)
+    BOOST_OCTONION_COMMON_GENERATOR_NEAR_EPS(exp)
     
     
     BOOST_OCTONION_TEST
@@ -746,7 +758,7 @@ boost::unit_test_framework::test_suite *    init_unit_test_suite(int, char *[])
 #undef    BOOST_OCTONION_TEST
     
 #undef    BOOST_OCTONION_COMMON_GENERATOR
-    
+#undef BOOST_OCTONION_COMMON_GENERATOR_NEAR_EPS
     
 #ifdef BOOST_OCTONION_TEST_VERBOSE
     
