@@ -8,8 +8,8 @@
 // See http://www.boost.org/libs/mpl for documentation.
 
 // $Source: /cvsroot/boost/boost/libs/mpl/test/bool.cpp,v $
-// $Date: 2004/09/02 15:41:35 $
-// $Revision: 1.3 $
+// $Date: 2006/02/20 15:45:07 $
+// $Revision: 1.4 $
 
 #include <boost/mpl/bool.hpp>
 #include <boost/mpl/aux_/test.hpp>
@@ -21,7 +21,7 @@
     { MPL_ASSERT(( is_same< bool_<c>, c##_ > )); } \
     { MPL_ASSERT(( is_same< bool_<c>::type, bool_<c> > )); } \
     { MPL_ASSERT_RELATION( bool_<c>::value, ==, c ); } \
-    assert( bool_<c>() == c ); \
+    BOOST_TEST( bool_<c>() == c ); \
 /**/
 
 MPL_TEST_CASE()
