@@ -1,6 +1,6 @@
 /* Boost.MultiIndex test for modifier memfuns.
  *
- * Copyright 2003-2005 Joaqun M Lpez Muoz.
+ * Copyright 2003-2006 Joaqun M Lpez Muoz.
  * Distributed under the Boost Software License, Version 1.0.
  * (See accompanying file LICENSE_1_0.txt or copy at
  * http://www.boost.org/LICENSE_1_0.txt)
@@ -16,6 +16,37 @@
 #include "employee.hpp"
 #include <boost/test/test_tools.hpp>
 
+class always_one
+{
+public:
+  always_one():n(1){}
+  ~always_one(){n=0;}
+
+  int get()const{return n;}
+
+private:
+  int n;
+};
+
+
+inline bool operator==(const always_one& x,const always_one& y)
+{
+  return x.get()==y.get();
+}
+
+#if defined(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP)
+namespace boost{
+#endif
+
+inline std::size_t hash_value(const always_one& x)
+{
+  return static_cast<std::size_t>(x.get());
+}
+
+#if defined(BOOST_NO_ARGUMENT_DEPENDENT_LOOKUP)
+} /* namespace boost */
+#endif
+
 using namespace boost::multi_index;
 
 void test_modifiers()
@@ -25,6 +56,7 @@ void test_modifiers()
   employee_set_by_age&      i2=get<age>(es);
   employee_set_as_inserted& i3=get<as_inserted>(es);
   employee_set_by_ssn&      i4=get<ssn>(es);
+  employee_set_randomly&    i5=get<randomly>(es);
 
   es.insert(employee(0,"Joe",31,1123));
   BOOST_CHECK(es.insert(employee(0,"Joe",31,1123)).second==false);
@@ -45,27 +77,36 @@ void test_modifiers()
 
   employee_set_as_inserted::iterator it3=i3.begin();
   i3.insert(it3,100,employee(3,"Judy",39,6201));
+  BOOST_CHECK((--it3)->ssn==6201);
   BOOST_CHECK(es.size()==4);
 
+  employee_set_randomly::iterator it5=i5.begin();
+  i5.insert(it5,100,employee(4,"Jill",52,3379));
+  BOOST_CHECK(i5.begin()->age==52);
+  BOOST_CHECK(es.size()==5);
+
   es.erase(employee(1,"Joe Jr.",5,2563));
-  BOOST_CHECK(i2.size()==3&&i3.size()==3);
+  BOOST_CHECK(i3.size()==4&&i5.size()==4);
 
-  i1.erase("Judy");
-  BOOST_CHECK(es.size()==2&&i2.size()==2);
+  BOOST_CHECK(i1.erase("Judy")==1);
+  BOOST_CHECK(es.size()==3&&i2.size()==3);
 
-  i2.erase(it2);
-  BOOST_CHECK(es.size()==1&&i1.size()==1);
+  BOOST_CHECK(i2.erase(it2)->age==52);
+  BOOST_CHECK(i3.size()==2&&i4.size()==2);
 
   i3.pop_front();
-  BOOST_CHECK(es.size()==0&&i2.size()==0);
+  BOOST_CHECK(i1.size()==1&&i2.size()==1);
+
+  i5.erase(i5.begin(),i5.end());
+  BOOST_CHECK(es.size()==0&&i3.size()==0);
 
   es.insert(employee(0,"Joe",31,1123));
-  i1.erase(i1.begin());
+  BOOST_CHECK(i1.erase(i1.begin())==i1.end());
   BOOST_CHECK(i1.size()==0);
 
   es.insert(employee(0,"Joe",31,1123));
   es.insert(employee(1,"Jack",31,5032));
-  i2.erase(31);
+  BOOST_CHECK(i2.erase(31)==2);
   BOOST_CHECK(i2.size()==0);
 
   i3.push_front(employee(1,"Jack",31,5032));
@@ -80,6 +121,27 @@ void test_modifiers()
   i3.pop_front();
   BOOST_CHECK(es.size()==0);
 
+  i5.push_back(employee(1,"Jack",31,5032));
+  i5.push_front(employee(0,"Joe",31,1123));
+  i5.insert(i5.end()-1,employee(2,"Grandda Joe",64,7881));
+  BOOST_CHECK(i5.back()==employee(1,"Jack",31,5032));
+  BOOST_CHECK(i5.front()==employee(0,"Joe",31,1123));
+  BOOST_CHECK(i5[0]==i5.front()&&i5.at(0)==i5.front());
+  BOOST_CHECK(i5[i5.size()-1]==i5.back()&&i5.at(i5.size()-1)==i5.back());
+
+  i5.pop_front();
+  BOOST_CHECK(i5.back()==employee(1,"Jack",31,5032));
+  BOOST_CHECK(i5.front()==employee(2,"Grandda Joe",64,7881));
+  BOOST_CHECK(es.size()==2);
+
+  i5.pop_back();
+  BOOST_CHECK(i5.back()==employee(2,"Grandda Joe",64,7881));
+  BOOST_CHECK(i5.front()==i5.front());
+  BOOST_CHECK(es.size()==1);
+
+  i5.erase(i5.begin());
+  BOOST_CHECK(es.size()==0);
+  
   std::vector<employee> ve;
   ve.push_back(employee(3,"Anna",31,5388));
   ve.push_back(employee(1,"Rachel",27,9012));
@@ -88,7 +150,7 @@ void test_modifiers()
   i1.insert(ve.begin(),ve.end());
   BOOST_CHECK(i2.size()==3);
 
-  i4.erase(i4.begin(),i4.end());
+  BOOST_CHECK(i2.erase(i2.begin(),i2.end())==i2.end());
   BOOST_CHECK(es.size()==0);
 
   i2.insert(ve.begin(),ve.end());
@@ -101,7 +163,20 @@ void test_modifiers()
   i3.insert(i3.end(),ve.begin(),ve.end());
   BOOST_CHECK(es.size()==3);
 
-  es.erase(es.begin(),es.end());
+  BOOST_CHECK(i4.erase(9012)==1);
+  i4.erase(i4.begin());
+  BOOST_CHECK(i4.erase(i4.begin(),i4.end())==i4.end());
+
+  i4.insert(ve.begin(),ve.end());
+  BOOST_CHECK(i5.size()==3);
+
+  BOOST_CHECK(i5.erase(i5.begin(),i5.end())==i5.end());
+  BOOST_CHECK(es.size()==0);
+
+  i5.insert(i5.begin(),ve.begin(),ve.end());
+  BOOST_CHECK(i1.size()==3);
+
+  BOOST_CHECK(es.erase(es.begin(),es.end())==es.end());
   BOOST_CHECK(i2.size()==0);
 
   es.insert(employee(0,"Joe",31,1123));
@@ -128,6 +203,12 @@ void test_modifiers()
   i3.swap(get<3>(es2));
   BOOST_CHECK(es==es2_backup&&es2==es_backup);
 
+  i4.swap(get<4>(es2));
+  BOOST_CHECK(es==es_backup&&es2==es2_backup);
+
+  i5.swap(get<5>(es2));
+  BOOST_CHECK(es==es2_backup&&es2==es_backup);
+
 #if defined(BOOST_FUNCTION_SCOPE_USING_DECLARATION_BREAKS_ADL)
   ::boost::multi_index::detail::swap(i1,get<1>(es2));
 #else
@@ -155,6 +236,24 @@ void test_modifiers()
 
   BOOST_CHECK(es==es_backup&&es2==es2_backup);
 
+#if defined(BOOST_FUNCTION_SCOPE_USING_DECLARATION_BREAKS_ADL)
+  ::boost::multi_index::detail::swap(i4,get<4>(es2));
+#else
+  using std::swap;
+  swap(i4,get<4>(es2));
+#endif
+
+  BOOST_CHECK(es==es2_backup&&es2==es_backup);
+
+#if defined(BOOST_FUNCTION_SCOPE_USING_DECLARATION_BREAKS_ADL)
+  ::boost::multi_index::detail::swap(i5,get<5>(es2));
+#else
+  using std::swap;
+  swap(i5,get<5>(es2));
+#endif
+
+  BOOST_CHECK(es==es_backup&&es2==es2_backup);
+
   i3.clear();
   BOOST_CHECK(i3.size()==0);
 
@@ -162,6 +261,26 @@ void test_modifiers()
   i4.clear();
   BOOST_CHECK(i4.size()==0);
 
+  es=es2;
+  i5.clear();
+  BOOST_CHECK(i5.size()==0);
+
   es2.clear();
   BOOST_CHECK(es2.size()==0);
+
+  /* testcase for problem reported at
+   * http://lists.boost.org/boost-users/2006/12/24215.php
+   */
+
+  multi_index_container<
+    always_one,
+    indexed_by<
+      hashed_non_unique<identity<always_one> >
+    >
+  > aoc;
+
+  aoc.insert(always_one());
+  aoc.insert(always_one());
+  aoc.erase(*(aoc.begin()));
+  BOOST_CHECK(aoc.empty());
 }
