@@ -20,7 +20,15 @@ void test_ptr_deque()
     reversible_container_test< ptr_deque< nullable<Base> >, Base, Derived_class >();
     reversible_container_test< ptr_deque< nullable<Value> >, Value, Value >();
 
+    test_transfer< ptr_deque<Derived_class>, ptr_deque<Base>, Derived_class>();
+
+    
     random_access_algorithms_test< ptr_deque<int> >();
+    ptr_deque<int> di;
+    di.push_front( new int(0) );
+    BOOST_CHECK_EQUAL( di.size(), 1u );
+    di.push_front( std::auto_ptr<int>( new int(1) ) );
+    BOOST_CHECK_EQUAL( di.size(), 2u ); 
 }
 
 using boost::unit_test::test_suite;
