@@ -4,7 +4,12 @@
 # http:#www.boost.org/LICENSE_1_0.txt)
 
 from declarations import *
-from elementtree.ElementTree import ElementTree
+# try to use cElementTree if avaiable
+try:
+    from cElementTree import ElementTree    
+except ImportError:
+    # fall back to the normal elementtree
+    from elementtree.ElementTree import ElementTree
 from xml.parsers.expat import ExpatError
 from copy import deepcopy
 from utils import enumerate
@@ -403,12 +408,8 @@ class GCCXMLParser(object):
         classname = self.GetDecl(element.get('context')).FullName()
         location = self.GetLocation(element.get('location'))
         params = self.GetArguments(element)
-        artificial = element.get('artificial', False)
-        if not artificial:
-            ctor = Constructor(name, classname, params, visib)
-        else:
-            # we don't want artificial constructors
-            ctor = Unknown('__Unknown_Element_%s' % id)
+        artificial = element.get('artificial', False)        
+        ctor = Constructor(name, classname, params, visib)
         ctor.location = location
         self.Update(id, ctor)
 
