@@ -2,12 +2,14 @@
 // Distributed under the Boost Software License, Version 1.0. (See
 // accompanying file LICENSE_1_0.txt or copy at
 // http://www.boost.org/LICENSE_1_0.txt)
+
 #include <boost/python/class.hpp>
 #include <boost/python/module.hpp>
 #include <boost/python/def.hpp>
 #include <boost/python/call_method.hpp>
 #include <boost/ref.hpp>
 #include <boost/utility.hpp>
+#include <boost/assert.hpp>
 
 using namespace boost::python;
 
@@ -15,10 +17,10 @@ struct X
 {
     explicit X(int x) : x(x), magic(7654321) { ++counter; }
     X(X const& rhs) : x(rhs.x), magic(7654321) { ++counter; }
-    virtual ~X() { assert(magic == 7654321); magic = 6666666; x = 9999; --counter; }
+    virtual ~X() { BOOST_ASSERT(magic == 7654321); magic = 6666666; x = 9999; --counter; }
 
-    void set(int x) { assert(magic == 7654321); this->x = x; }
-    int value() const { assert(magic == 7654321); return x; }
+    void set(int x) { BOOST_ASSERT(magic == 7654321); this->x = x; }
+    int value() const { BOOST_ASSERT(magic == 7654321); return x; }
     static int count() { return counter; }
  private:
     void operator=(X const&);
