@@ -5,7 +5,7 @@
  * accompanying file LICENSE_1_0.txt or copy at
  * http://www.boost.org/LICENSE_1_0.txt)
  *
- * $Id: random_demo.cpp,v 1.16 2004/07/27 03:43:34 dgregor Exp $
+ * $Id: random_demo.cpp,v 1.17 2005/05/10 20:40:59 jmaurer Exp $
  *
  * A short demo program how to use the random number library.
  */
@@ -40,7 +40,7 @@ void experiment(base_generator_type & generator)
   // Define a uniform random number distribution of integer values between
   // 1 and 6 inclusive.
   typedef boost::uniform_int<> distribution_type;
-  typedef boost::variate_generator<base_generator_type, distribution_type> gen_type;
+  typedef boost::variate_generator<base_generator_type&, distribution_type> gen_type;
   gen_type die_gen(generator, distribution_type(1, 6));
 
 #if !defined(__SUNPRO_CC) || (__SUNPRO_CC > 0x530)
