@@ -1,7 +1,7 @@
 /*
  *
  * Copyright (c) 1998-2002
- * Dr John Maddock
+ * John Maddock
  *
  * Use, modification and distribution are subject to the 
  * Boost Software License, Version 1.0. (See accompanying file 
@@ -44,6 +44,13 @@ int test_main( int , char* [] )
    BOOST_CHECK_THROW(boost::regex_search(bad_text, what, e2), std::runtime_error);
    BOOST_CHECK(boost::regex_search(good_text, what, e2));
 
+   bad_text.assign((std::string::size_type)500000, 'a');
+   e2.assign("aaa*@");
+   BOOST_CHECK_THROW(0 == boost::regex_search(bad_text, what, e2), std::runtime_error);
+   good_text.assign((std::string::size_type)5000, 'a');
+   BOOST_CHECK(0 == boost::regex_search(good_text, what, e2));
+
    return 0;
 }
 
+#include <boost/test/included/test_exec_monitor.hpp>
