@@ -111,6 +111,8 @@ void_caster_registry * void_caster_registry::m_self = NULL;
 
 void 
 void_caster_registry::purge(const extended_type_info * eti){
+    if(NULL == m_self)
+        return;
     if(! empty()){
         iterator i = m_self->m_set.begin();
         while(i != m_self->m_set.end()){
@@ -171,12 +173,12 @@ public:
 class void_caster_argument : public void_caster
 {
     virtual void const*
-    upcast( void const* t ) const {
+    upcast( void const* /* t */ ) const {
         assert(false);
         return NULL;
     }
     virtual void const*
-    downcast( void const* t ) const {
+    downcast( void const* /* t */ ) const {
         assert(false);
         return NULL;
     }
