@@ -73,7 +73,24 @@ namespace archive {
 } // archive
 } // boost
 
-#else
+#else // defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
+#if defined(__hpux)
+// (C) Copyright 2006 Boris Gubenko.
+// HP-UX has a restriction that for multi-thread applications, (i.e.
+// the ones compiled -mt) if argument to tmpnam is a NULL pointer, then,
+// citing the tmpnam(3S) manpage, "the operation is not performed and a
+// NULL pointer is returned". tempnam does not have this restriction, so,
+// let's use tempnam instead.
+ 
+#define tmpnam(X) tempnam(NULL,X)
+ 
+namespace boost {
+namespace archive {
+    using ::tempnam;
+} // archive
+} // boost
+
+#else // defined(__hpux)
 
 namespace boost {
 namespace archive {
@@ -81,6 +98,7 @@ namespace archive {
 } // archive
 } // boost
 
+#endif // defined(__hpux)
 #endif // defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
 
 /////////////////////////////////////////////
