@@ -23,7 +23,7 @@ void increment_action_test()
 
     const char* cp = "63";
     const char* cp_first = cp;
-    const char* cp_last = cp + string_length(cp);
+    const char* cp_last = cp + test_impl::string_length(cp);
     int h=127;
     int hm=h;
 
