@@ -7,7 +7,7 @@
     http://www.boost.org/LICENSE_1_0.txt)
 =============================================================================*/
 #include <iostream>
-#include <cassert>
+#include <boost/detail/lightweight_test.hpp>
 #include <string>
 
 using namespace std;
@@ -28,43 +28,43 @@ void
 match_tests()
 {
     match<> m0;
-    assert(!m0.has_valid_attribute());
+    BOOST_TEST(!m0.has_valid_attribute());
 
     match<int> m1(m0);
     m1.value(123);
-    assert(m1.has_valid_attribute());
-    assert(m1.value() == 123);
+    BOOST_TEST(m1.has_valid_attribute());
+    BOOST_TEST(m1.value() == 123);
 
     match<double> m2(m1);
-    assert(m2.has_valid_attribute());
-    assert(m1.value() == int(m2.value()));
+    BOOST_TEST(m2.has_valid_attribute());
+    BOOST_TEST(m1.value() == int(m2.value()));
     m2.value(456);
 
     m0 = m0; // match<nil> = match<nil>
     m0 = m1; // match<nil> = match<int>
     m0 = m2; // match<nil> = match<double>
     m1 = m0; // match<int> = match<nil>
-    assert(!m1);
-    assert(!m1.has_valid_attribute());
+    BOOST_TEST(!m1);
+    BOOST_TEST(!m1.has_valid_attribute());
 
     m1 = m1; // match<int> = match<int>
     m1.value(int(m2.value()));
-    assert(m1.has_valid_attribute());
-    assert(m1.value() == int(m2.value()));
+    BOOST_TEST(m1.has_valid_attribute());
+    BOOST_TEST(m1.value() == int(m2.value()));
 
     m2.value(123.456);
     match<Z> mz(m2); // copy from match<double>
     mz = m2; // assign from match<double>
-    assert(mz.value().n == 123.456);
+    BOOST_TEST(mz.value().n == 123.456);
 
     m1.value(123);
     m2 = m0;
-    assert(!m2);
-    assert(!m2.has_valid_attribute());
+    BOOST_TEST(!m2);
+    BOOST_TEST(!m2.has_valid_attribute());
 
     m2 = m1; // match<double> = match<int>
-    assert(m2.has_valid_attribute());
-    assert(m1.value() == int(m2.value()));
+    BOOST_TEST(m2.has_valid_attribute());
+    BOOST_TEST(m1.value() == int(m2.value()));
     m2 = m2; // match<double> = match<double>
 
     cout << "sizeof(int) == " << sizeof(int) << '\n';
@@ -73,58 +73,66 @@ match_tests()
     cout << "sizeof(match<double>) == " << sizeof(m2) << '\n';
 
     match<int&> mr;
-    assert(!mr.has_valid_attribute());
+    BOOST_TEST(!mr.has_valid_attribute());
 
     match<int&> mrr(4);
-    assert(!mrr.has_valid_attribute());
+    BOOST_TEST(!mrr.has_valid_attribute());
 
     int ri = 3;
     match<int&> mr2(4, ri);
-    assert(mr2.has_valid_attribute());
+    BOOST_TEST(mr2.has_valid_attribute());
     mr = mr2;
-    assert(mr.has_valid_attribute());
+    BOOST_TEST(mr.has_valid_attribute());
 
     match<int&> mr3(mrr);
-    assert(!mr3.has_valid_attribute());
+    BOOST_TEST(!mr3.has_valid_attribute());
     mr2 = mr3;
-    assert(!mr2.has_valid_attribute());
+    BOOST_TEST(!mr2.has_valid_attribute());
 
     match<X> mx;
     m1 = mx;
     m0 = mx;
-    assert(!mx.has_valid_attribute());
-    assert(!m0.has_valid_attribute());
-    assert(!m1.has_valid_attribute());
+    BOOST_TEST(!mx.has_valid_attribute());
+    BOOST_TEST(!m0.has_valid_attribute());
+    BOOST_TEST(!m1.has_valid_attribute());
 
     match<Y> my;
-    assert(!my.has_valid_attribute());
+    BOOST_TEST(!my.has_valid_attribute());
 
     match<std::string> ms;
-    assert(!ms.has_valid_attribute());
+    BOOST_TEST(!ms.has_valid_attribute());
     ms.value("Kimpo Ponchwayla");
-    assert(ms.has_valid_attribute());
-    assert(ms.value() == "Kimpo Ponchwayla");
+    BOOST_TEST(ms.has_valid_attribute());
+    BOOST_TEST(ms.value() == "Kimpo Ponchwayla");
     ms = match<>();
-    assert(!ms.has_valid_attribute());
+    BOOST_TEST(!ms.has_valid_attribute());
 
     // let's try a match with a reference:
     int i;
     match<int&> mr4(4, i);
-    assert(mr4.has_valid_attribute());
+    BOOST_TEST(mr4.has_valid_attribute());
     mr4.value(3);
-    assert(mr4.value() == 3);
-    assert(i == 3);
+    BOOST_TEST(mr4.value() == 3);
+    BOOST_TEST(i == 3);
     (void)i;
 
     int x = 456;
     match<int&> mref(1, x);
-    assert(mref.value() == 456);
+    BOOST_TEST(mref.value() == 456);
     mref.value(123);
-    assert(mref.value() == 123);
+    BOOST_TEST(mref.value() == 123);
     x = mref.value();
-    assert(x == 123);
+    BOOST_TEST(x == 123);
     mref.value() = 986;
-    assert(x == 986);
+    BOOST_TEST(x == 986);
+    
+    std::string s("hello");
+    match<int> mint(1, x);
+    BOOST_TEST(mint.value() == x);
+    match<std::string> mstr(1, s);
+    BOOST_TEST(mstr.value() == "hello");
+    mstr = mint;
+    mint = mstr;
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -140,16 +148,16 @@ match_policy_tests()
     match<double>   m2;
     match_policy    mp;
 
-    m0 = mp.no_match();     assert(!m0);
-    m1 = mp.no_match();     assert(!m1);
-    m0 = mp.empty_match();  assert(!!m0);
-    m2 = mp.empty_match();  assert(!!m2);
+    m0 = mp.no_match();     BOOST_TEST(!m0);
+    m1 = mp.no_match();     BOOST_TEST(!m1);
+    m0 = mp.empty_match();  BOOST_TEST(!!m0);
+    m2 = mp.empty_match();  BOOST_TEST(!!m2);
 
     m1 = mp.create_match(5, 100, 0, 0);
     m2 = mp.create_match(5, 10.5, 0, 0);
 
     mp.concat_match(m1, m2);
-    assert(m1.length() == 10);
+    BOOST_TEST(m1.length() == 10);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -162,7 +170,6 @@ main()
 {
     match_tests();
     match_policy_tests();
-    cout << "Tests concluded successfully\n";
-    return 0;
+    return boost::report_errors();
 }
 
