@@ -10,6 +10,7 @@
 #include "boost/filesystem/operations.hpp"
 #include "boost/filesystem/fstream.hpp"
 #include "boost/filesystem/exception.hpp"
+#include "boost/filesystem/convenience.hpp"
 
 #include <iostream>
 #include <string>
@@ -26,6 +27,12 @@ namespace fs = boost::filesystem;
 #define BOOST_NO_CPP_MAIN_SUCCESS_MESSAGE
 #include <boost/test/included/prg_exec_monitor.hpp>
 
+// options 
+
+static bool echo = false;
+static bool create_dirs = false;
+static bool boost_build_v2 = false;
+
 namespace
 {
   struct test_info
@@ -82,6 +89,33 @@ namespace
       if ( *itr == '\\' || *itr == '!' ) *itr = '/';
   }
 
+//  trim_left ----------------------------------------------------------------//
+
+  std::string trim_left( std::string const& s )
+  {
+    std::string::size_type const pos( s.find_first_not_of(' ') );
+    return pos != std::string::npos
+        ? s.substr( pos, s.size() - pos + 1 )
+        : ""
+        ;
+  }
+  
+
+//  split --------------------------------------------------------------------//
+
+  std::vector<std::string> split( std::string const& s )
+  {
+    std::string::size_type const pos( s.find_first_of(' ') );
+    std::vector<std::string> result( 1, s.substr( 0, pos ) );
+    if ( pos == std::string::npos )
+        return result;
+
+    std::vector<std::string> const rest( split( trim_left( s.substr( pos, s.size() - pos + 1 ) ) ) );
+    result.insert( result.end(), rest.begin(), rest.end() );    
+    return result;
+  }
+
+
 //  extract a target directory path from a jam target string  ----------------//
 //  s may be relative to the initial_path:
 //    ..\..\..\libs\foo\build\bin\libfoo.lib\vc7\debug\runtime-link-dynamic\boo.obj
@@ -95,11 +129,11 @@ namespace
     string temp( s );
     convert_path_separators( temp );
     temp.erase( temp.find_last_of( "/" ) ); // remove leaf
-    string::size_type pos = temp.find_last_of( " " ); // remove leading spaces
-    if ( pos != string::npos ) temp.erase( 0, pos+1 );
+    temp = split( trim_left( temp ) ).back();
     if ( temp[0] == '.' ) temp.erase( 0, temp.find_first_not_of( "./" ) ); 
     else temp.erase( 0, locate_root.string().size()+1 );
-//std::cout << "\"" << s << "\", \"" << temp << "\"" << std::endl;
+    if ( echo )
+        std::cout << "\ttarget_directory( \"" << s << "\") -> \"" << temp << "\"" << std::endl;
     return temp;
   }
 
@@ -110,12 +144,14 @@ namespace
     if ( pos == string::npos ) pos = s.find( ".so/" );
     if ( pos == string::npos ) pos = s.find( ".lib/" );
     if ( pos == string::npos ) pos = s.find( ".pyd/" );
+    if ( pos == string::npos ) pos = s.find( ".a/" );
     return pos;
   }
 
   string toolset( const string & s )
   {
     string::size_type pos = target_name_end( s );
+    if ( pos == string::npos ) pos = s.find( "build/" );
     if ( pos == string::npos ) return "";
     pos = s.find( "/", pos ) + 1;
     return s.substr( pos, s.find( "/", pos ) - pos );
@@ -131,26 +167,113 @@ namespace
         ? pos : s.find( "/", pos )) - pos_start );
   }
 
+  // Take a path to a target directory of test, and
+  // returns library name corresponding to that path.
   string test_path_to_library_name( string const& path )
   {
     std::string result;
     string::size_type start_pos( path.find( "libs/" ) );
     if ( start_pos != string::npos )
     {
-      start_pos += 5;
-      string::size_type end_pos( path.find( '/', start_pos ) );
-      result = path.substr( start_pos, end_pos - start_pos );
+      // The path format is ...libs/functional/hash/test/something.test/....      
+      // So, the part between "libs" and "test/something.test" can be considered
+      // as library name. But, for some libraries tests are located too deep,
+      // say numeric/ublas/test/test1 directory, and some libraries have tests
+      // in several subdirectories (regex/example and regex/test). So, nested
+      // directory may belong to several libraries.
 
-      // if a "sublibs" file exists, the library name includes the
-      // next level down directory name.
-      if ( fs::exists( ( boost_root / "libs" ) / result / "sublibs" ) )
+      // To disambituate, it's possible to place a 'sublibs' file in
+      // a directory. It means that child directories are separate libraries.
+      // It's still possible to have tests in the directory that has 'sublibs'
+      // file.
+
+      std::string interesting;
+      start_pos += 5;
+      string::size_type end_pos( path.find( ".test/", start_pos ) );
+      end_pos = path.rfind('/', end_pos);
+      if (path.substr(end_pos - 5, 5) == "/test")
+        interesting = path.substr( start_pos, end_pos - 5 - start_pos );
+      else
+        interesting = path.substr( start_pos, end_pos - start_pos );
+
+      // Take slash separate elements until we have corresponding 'sublibs'.
+      end_pos = 0;
+      for(;;)
       {
-        result += path.substr( end_pos, path.find( '/', end_pos+1 ) - end_pos );
+        end_pos = interesting.find('/', end_pos);
+        if (end_pos == string::npos) {
+          result = interesting;
+          break;
+        }
+        result = interesting.substr(0, end_pos);
+
+        if ( fs::exists( ( boost_root / "libs" ) / result / "sublibs" ) )
+        {
+          end_pos = end_pos + 1;
+        }
+        else
+          break;
       }
     }
 
     return result;
   }
+
+  // Tries to find target name in the string 'msg', starting from 
+  // position start.
+  // If found, extract the directory name from the target name and
+  // stores it in 'dir', and return the position after the target name.
+  // Otherwise, returns string::npos.
+  string::size_type parse_skipped_msg_aux(const string& msg,
+                                          string::size_type start,
+                                          string& dir)
+  {
+    dir.clear();
+    string::size_type start_pos = msg.find( '<', start );
+    if ( start_pos == string::npos ) return string::npos;
+    ++start_pos;
+    string::size_type end_pos = msg.find( '>', start_pos );
+    dir += msg.substr( start_pos, end_pos - start_pos );
+    if ( boost_build_v2 )
+    {
+        // The first letter is a magic value indicating
+        // the type of grist.
+        convert_path_separators( dir );
+        dir.erase( 0, 1 );
+        // We need path from root, not from 'status' dir.
+        if (dir.find("../") == 0)
+          dir.erase(0,3);
+        else // dir is always relative to the boost directory tree
+          dir.erase( 0, locate_root.string().size()+1 );
+    }
+    else
+    {
+      if ( dir[0] == '@' )
+      {
+        // new style build path, rooted build tree
+        convert_path_separators( dir );
+        dir.replace( 0, 1, "bin/" );
+      }
+      else
+      {
+        // old style build path, integrated build tree
+        start_pos = dir.rfind( '!' );
+        convert_path_separators( dir );
+        string::size_type path_sep_pos = dir.find( '/', start_pos + 1 );
+        if ( path_sep_pos != string::npos )
+           dir.insert( path_sep_pos, "/bin" );
+        else
+        {
+          // see http://article.gmane.org/gmane.comp.lib.boost.devel/146688;
+          // the following code assumes that: a) 'dir' is not empty,
+          // b) 'end_pos != string::npos' and c) 'msg' always ends with '...'
+          if ( dir[dir.size() - 1] == '@' )
+            dir += "/" + msg.substr( end_pos + 1, msg.size() - end_pos - 1 - 3 );
+        }
+      }
+    }
+    return end_pos;
+  }
   
   // the format of paths is really kinky, so convert to normal form
   //   first path is missing the leading "..\".
@@ -161,47 +284,10 @@ namespace
   void parse_skipped_msg( const string & msg,
     string & first_dir, string & second_dir )
   {
-    first_dir.clear();
-    second_dir.clear();
-    string::size_type start_pos( msg.find( '<' ) );
-    if ( start_pos == string::npos ) return;
-    ++start_pos;
-    string::size_type end_pos( msg.find( '>', start_pos ) );
-    first_dir += msg.substr( start_pos, end_pos - start_pos );
-    if ( first_dir[0] == '@' )
-    {
-      // new style build path, rooted build tree
-      convert_path_separators( first_dir );
-      first_dir.replace( 0, 1, "bin/" );
-    }
-    else
-    {
-      // old style build path, integrated build tree
-      start_pos = first_dir.rfind( '!' );
-      convert_path_separators( first_dir );
-      first_dir.insert( first_dir.find( '/', start_pos + 1), "/bin" );
-    }
-//std::cout << first_dir << std::endl;
-
-    start_pos = msg.find( '<', end_pos );
-    if ( start_pos == string::npos ) return;
-    ++start_pos;
-    end_pos = msg.find( '>', start_pos );
-    second_dir += msg.substr( start_pos, end_pos - start_pos );
-    if ( second_dir[0] == '@' )
-    {
-      // new style build path, rooted build tree
-      convert_path_separators( second_dir );
-      second_dir.replace( 0, 1, "bin/" );
-    }
-    else
-    {
-      // old style build path, integrated build tree
-      start_pos = second_dir.rfind( '!' );
-      convert_path_separators( second_dir );
-      second_dir.insert( second_dir.find( '/', start_pos + 1), "/bin" );
-    }
-//std::cout << second_dir << std::endl;
+    string::size_type pos = parse_skipped_msg_aux(msg, 0, first_dir);
+    if (pos == string::npos)
+      return;
+    parse_skipped_msg_aux(msg, pos, second_dir);
   }
 
 //  test_log hides database details  -----------------------------------------//
@@ -251,6 +337,8 @@ namespace
         if ( target_directory.find( ".lib/" ) != string::npos
           || target_directory.find( ".dll/" ) != string::npos 
           || target_directory.find( ".so/" ) != string::npos 
+          || target_directory.find( ".dylib/" ) != string::npos 
+          || target_directory.find( "/build/" ) != string::npos 
           )
         {
           info.type = "lib";
@@ -280,6 +368,8 @@ namespace
     ~test_log()
     {
       fs::path pth( locate_root / m_target_directory / "test_log.xml" );
+      if ( create_dirs && !fs::exists( pth.branch_path() ) )
+          fs::create_directories( pth.branch_path() );
       fs::ofstream file( pth );
       if ( !file )
       {
@@ -347,12 +437,15 @@ namespace
                       const string & toolset,
                       const string & prior_content )
     {
+      assert( !target_directory.empty() );
+
       if ( !m_action_name.empty() ) stop_message( prior_content );
       m_action_name = action_name;
       m_target_directory = target_directory;
       m_test_name = test_name;
       m_toolset = toolset;
       m_note = false;
+
       if ( m_previous_target_directory != target_directory )
       {
         m_previous_target_directory = target_directory;
@@ -406,14 +499,17 @@ namespace
           tl.remove_action( "run" );
           if ( result == "fail" ) m_compile_failed = true;
         }
-        else if ( action_name == "link" ) { tl.remove_action( "run" ); }
+        else if ( action_name == "link" )
+        {
+          tl.remove_action( "run" );
+        }
 
         // dependency removal won't work right with random names, so assert
         else { assert( action_name == "run" ); }
 
         // add the "run" stop_message action
         tl.add_action( action_name,
-          result == "succeed" && note() ? "note" : result,
+           result == "succeed" && note() ? std::string("note") : result,
           timestamp, content );
       }
 
@@ -423,14 +519,23 @@ namespace
   };
 }
 
+
 //  main  --------------------------------------------------------------------//
 
+
 int cpp_main( int argc, char ** argv )
 {
+  // Turn off synchronization with corresponding C standard library files. This
+  // gives a significant speed improvement on platforms where the standard C++
+  // streams are implemented using standard C files.
+  std::ios::sync_with_stdio(false);
+
   if ( argc <= 1 )
-    std::cout << "Usage: bjam [bjam-args] | process_jam_log [--echo] [locate-root]\n"
-                 "  locate-root is the same as the bjam ALL_LOCATE_TARGET\n"
-                 "  parameter, if any. Default is boost-root.\n";
+    std::cout << "Usage: bjam [bjam-args] | process_jam_log [--echo] [--create-directories] [--v2] [locate-root]\n"
+                 "locate-root         - the same as the bjam ALL_LOCATE_TARGET\n"
+                 "                      parameter, if any. Default is boost-root.\n"
+                 "create-directories  - if the directory for xml file doesn't exists - creates it.\n"
+                 "                      usually used for processing logfile on different machine\n";
 
   boost_root = fs::initial_path();
 
@@ -446,16 +551,34 @@ int cpp_main( int argc, char ** argv )
     return 1;
   }
 
-  bool echo = false;
+  boost_root.normalize();
+  
   if ( argc > 1 && std::strcmp( argv[1], "--echo" ) == 0 )
   {
     echo = true;
     --argc; ++argv;
   }
 
+
+  if (argc > 1 && std::strcmp( argv[1], "--create-directories" ) == 0 )
+  {
+      create_dirs = true;
+      --argc; ++argv;
+  } 
+
+  if ( argc > 1 && std::strcmp( argv[1], "--v2" ) == 0 )
+  {
+    boost_build_v2 = true;
+    --argc; ++argv;
+  }
+
+
   if (argc > 1)
   {
       locate_root = fs::path( argv[1], fs::native );
+      if ( !locate_root.is_complete() )
+        locate_root = ( fs::initial_path() / locate_root ).normalize();
+      
       --argc; ++argv;
   } 
   else
@@ -488,12 +611,26 @@ int cpp_main( int argc, char ** argv )
   //   * Calls stop_message() to stop capturing lines.
   //   * Capture lines if line capture on.
 
+  int line_num = 0;
   while ( std::getline( *input, line ) )
   {
-    if ( echo ) std::cout << line << "\n";
+    ++line_num;
+    
+    std::vector<std::string> const line_parts( split( line ) );
+    std::string const line_start( line_parts[0] != "...failed" 
+        ? line_parts[0]
+        : line_parts[0] + " " + line_parts[1]
+        );
+    
+    if ( echo )
+    {
+      std::cout
+        << "line " << line_num << ": " << line << "\n"
+        << "\tline_start: " << line_start << "\n";        
+    }
 
     // create map of test-name to test-info
-    if ( line.find( "boost-test(" ) == 0 )
+    if ( line_start.find( "boost-test(" ) == 0 )
     {
       string::size_type pos = line.find( '"' );
       string test_name( line.substr( pos+1, line.find( '"', pos+1)-pos-1 ) );
@@ -526,27 +663,38 @@ int cpp_main( int argc, char ** argv )
 
     // these actions represent both the start of a new action
     // and the end of a failed action
-    else if ( line.find( "C++-action " ) != string::npos
-      || line.find( "vc-C++ " ) != string::npos
-      || line.find( "C-action " ) != string::npos
-      || line.find( "Cc-action " ) != string::npos
-      || line.find( "vc-Cc " ) != string::npos
-      || line.find( "Link-action " ) != string::npos
-      || line.find( "vc-Link " ) != string::npos 
-      || line.find( ".compile.") != string::npos
-      || ( line.find( ".link") != string::npos &&
+    else if ( line_start.find( "C++-action" ) != string::npos
+      || line_start.find( "vc-C++" ) != string::npos
+      || line_start.find( "C-action" ) != string::npos
+      || line_start.find( "Cc-action" ) != string::npos
+      || line_start.find( "vc-Cc" ) != string::npos
+      || line_start.find( ".compile.") != string::npos
+      || line_start.find( "compile-") != string::npos
+      || line_start.find( "-compile") != string::npos
+      || line_start.find( "Link-action" ) != string::npos
+      || line_start.find( "vc-Link" ) != string::npos 
+      || line_start.find( "Archive-action" ) != string::npos
+      || line_start.find( ".archive") != string::npos
+      || ( line_start.find( ".link") != string::npos &&
            // .linkonce is present in gcc linker messages about
            // unresolved symbols. We don't have to parse those
-           line.find( ".linkonce" ) == string::npos )
+           line_start.find( ".linkonce" ) == string::npos )
     )
     {
-      string action( ( line.find( "Link-action " ) != string::npos
-        || line.find( "vc-Link " ) != string::npos 
-        || line.find( ".link") != string::npos)
-        ? "link" : "compile" );
-      if ( line.find( "...failed " ) != string::npos )
+      string action( ( line_start.find( "Link-action" ) != string::npos
+            || line_start.find( "vc-Link" ) != string::npos 
+            || line_start.find( "Archive-action" ) != string::npos
+            || line_start.find( ".archive") != string::npos
+            || line_start.find( ".link") != string::npos
+            )
+          ? "link" : "compile"
+        );
+      
+      if ( line_start.find( "...failed " ) != string::npos )
+      {
         mgr.stop_message( action, target_directory( line ),
           "fail", timestamp(), content );
+      }
       else
       {
         string target_dir( target_directory( line ) );
@@ -558,18 +706,18 @@ int cpp_main( int argc, char ** argv )
     }
 
     // these actions are only used to stop the previous action
-    else if ( line.find( "-Archive" ) != string::npos
-      || line.find( "MkDir" ) == 0 )
+    else if ( line_start.find( "-Archive" ) != string::npos
+      || line_start.find( "MkDir" ) == 0 )
     {
       mgr.stop_message( content );
       content.clear();
       capture_lines = false;
     }
 
-    else if ( line.find( "execute-test" ) != string::npos 
-             || line.find( "testing.capture-output" ) != string::npos )
+    else if ( line_start.find( "execute-test" ) != string::npos 
+             || line_start.find( "capture-output" ) != string::npos )
     {
-      if ( line.find( "...failed " ) != string::npos )
+      if ( line_start.find( "...failed " ) != string::npos )
       {
         mgr.stop_message( "run", target_directory( line ),
           "fail", timestamp(), content );
@@ -601,16 +749,17 @@ int cpp_main( int argc, char ** argv )
     }
 
     // bjam indicates some prior dependency failed by a "...skipped" message
-    else if ( line.find( "...skipped <" ) != string::npos && line.find( "<directory-grist>" ) == string::npos)
+    else if ( line_start.find( "...skipped" ) != string::npos 
+        && line.find( "<directory-grist>" ) == string::npos
+        )
     {
       mgr.stop_message( content );
       content.clear();
       capture_lines = false;
 
-      if ( line.find( " for lack of " ) != string::npos
-        && line.find( ".run for lack of " ) == string::npos )
+      if ( line.find( " for lack of " ) != string::npos )
       {
-        capture_lines = true;
+        capture_lines = ( line.find( ".run for lack of " ) == string::npos );
 
         string target_dir;
         string lib_dir;
@@ -629,9 +778,9 @@ int cpp_main( int argc, char ** argv )
 
     }
 
-    else if ( line.find( "**passed**" ) != string::npos
-      || line.find( "failed-test-file " ) != string::npos
-      || line.find( "command-file-dump" ) != string::npos )
+    else if ( line_start.find( "**passed**" ) != string::npos
+      || line_start.find( "failed-test-file" ) != string::npos
+      || line_start.find( "command-file-dump" ) != string::npos )
     {
       mgr.stop_message( content );
       content = "\n";
